@@ -6,11 +6,14 @@
 from collections import OrderedDict
 import copy
 import json
+import logging
 import os.path
 import pkg_resources
 
 from .utils import cotederc
 
+module_logger = logging.getLogger(__name__)
+
 
 def list_cfgs():
     """List the available QC procedures, builtin + local
@@ -24,7 +27,7 @@ def list_cfgs():
     --------
     utils.load_cfg
     """
-    cfg = pkg_resources.resource_listdir('cotede', "qc_cfg")
+    cfg = pkg_resources.resource_listdir("cotede", "qc_cfg")
     cfg = sorted([c[:-5] for c in cfg if c[-5:] == ".json"])
 
     ucfg = os.listdir(cotederc("cfg"))
@@ -95,28 +98,28 @@ def load_cfg(cfgname="cotede"):
     if cfgname is None:
         cfgname = "cotede"
 
-    assert type(cfgname) in (dict, str), \
-            'load_cfg() input must be a dictionary or a str'
+    assert isinstance(
+        cfgname, (dict, str)
+    ), "load_cfg() input must be a dictionary or a str"
 
     # A given manual configuration has priority
     if isinstance(cfgname, dict):
-        # self.logger.debug("%s - User's QC cfg." % self.name)
+        module_logger.debug("User's QC cfg: %s" % cfgname)
         cfg = OrderedDict(copy.deepcopy(cfgname))
     elif isinstance(cfgname, str):
         try:
-            # If cfg is available in qc_cfg, use it
+            # If cfg is available in builtin options, use it
             p = pkg_resources.resource_string(
                 "cotede", os.path.join("qc_cfg", "{}.json".format(cfgname))
             )
             cfg = json.loads(p, object_pairs_hook=OrderedDict)
-            # self.logger.debug("%s - QC cfg: %s" % (self.name, cfg))
+            module_logger.debug("Builtin config - %s" % cfgname)
         except:
             # Otherwise, try to load from user's directory
-            p = os.path.join(cotederc(), "cfg", "{}.json".format(cfgname))
-            with open(p, 'r') as f:
+            p = os.path.join(cotederc("cfg"), "{}.json".format(cfgname))
+            with open(p, "r") as f:
                 cfg = json.load(f, object_pairs_hook=OrderedDict)
-        # self.logger.debug("%s - QC cfg: ~/.cotederc/%s" %
-        #            (self.name, cfg))
+            module_logger.debug("User collection cfg - %s" % cfgname)
 
     cfg = fix_config(cfg)
     if "inherit" in cfg:
@@ -125,6 +128,8 @@ def load_cfg(cfgname="cotede"):
         for parent in cfg["inherit"]:
             cfg = inheritance(cfg, load_cfg(parent))
 
+    cfg = fix_procedure(cfg)
+
     return cfg
 
 
@@ -134,10 +139,10 @@ def fix_config(cfg):
        This function allows backward compatibility with old config descriptos
        updating them to the current standard.
     """
-    if ('revision' in cfg) and (cfg['revision'] == '0.21'):
+    if ("revision" in cfg) and (cfg["revision"] == "0.21"):
         return cfg
 
-    if 'revision' not in cfg:
+    if "revision" not in cfg:
         cfg = convert_pre_to_021(cfg)
 
     return cfg
@@ -153,33 +158,34 @@ def convert_pre_to_021(cfg):
          - TEMP -> sea_water_temperature
          - PSAL -> sea_water_salinity
     """
+
     def label(v):
         """Convert Ocean Sites vocabulary to CF standard names
         """
-        if v == 'PRES':
-            return 'sea_water_pressure'
-        if v == 'TEMP':
-            return 'sea_water_temperature'
-        elif v == 'PSAL':
-            return 'sea_water_salinity'
+        if v == "PRES":
+            return "sea_water_pressure"
+        if v == "TEMP":
+            return "sea_water_temperature"
+        elif v == "PSAL":
+            return "sea_water_salinity"
         else:
             return v
 
     keys = list(cfg.keys())
 
     output = OrderedDict()
-    output['revision'] = '0.21'
+    output["revision"] = "0.21"
 
-    if 'inherit' in keys:
-        output['inherit'] = cfg['inherit']
-        keys.remove('inherit')
+    if "inherit" in keys:
+        output["inherit"] = cfg["inherit"]
+        keys.remove("inherit")
 
-    if 'main' in cfg:
-        output['common'] = cfg['main']
-        keys.remove('main')
-    elif 'common' in cfg:
-        output['common'] = cfg['common']
-        keys.remove('common')
+    if "main" in cfg:
+        output["common"] = cfg["main"]
+        keys.remove("main")
+    elif "common" in cfg:
+        output["common"] = cfg["common"]
+        keys.remove("common")
 
     def fix_threshold(cfg):
         """Explicit threshold"""
@@ -206,12 +212,82 @@ def convert_pre_to_021(cfg):
             cfg["profile_envelop"] = {"layers": cfg["profile_envelop"]}
         return cfg
 
-    output['variables'] = OrderedDict()
+    output["variables"] = OrderedDict()
     for k in keys:
         cfg[k] = fix_threshold(cfg[k])
         cfg[k] = fix_regional_range(cfg[k])
         cfg[k] = fix_profile_envelop(cfg[k])
-        output['variables'][label(k)] = cfg[k]
-        # output[k] = cfg[k]
+        output["variables"][label(k)] = cfg[k]
 
     return output
+
+
+def guess_procedure(name, cfg=None):
+    catalog = {
+        "anomaly_detection": None,
+        "bin_spike": "Bin_Spike",
+        "cars_normbias": "CARS_NormBias",
+        "constant_cluster_size": "ConstantClusterSize",
+        "cum_rate_of_change": "CumRateOfChange",
+        "deepest_pressure": "DeepestPressure",
+        "density_inversion": "DensityInversion",
+        "digit_roll_over": "DigitRollOver",
+        "fuzzylogic": None,
+        "frozen_profile": None,
+        "global_range": "GlobalRange",
+        "gradient": "Gradient",
+        "gradient_depthconditional": "GradientDepthConditional",
+        "grey_list": None,
+        "gross_sensor_drift": None,
+        "monotonic_z": "MonotonicZ",
+        "morello2014": None,
+        "platform_identification": None,
+        "pressure_increasing": None,
+        "profile_envelop": "ProfileEnvelop",
+        "pstep": None,
+        "rate_of_change": "RateOfChange",
+        "regional_range": "RegionalRange",
+        "spike": "Spike",
+        "spike_depthconditional": "SpikeDepthConditional",
+        "stuck_value": "StuckValue",
+        "tukey53H": "Tukey53H",
+        "tukey53H_norm": "Tukey53H",
+        "valid_speed": None,
+        "woa_normbias": "WOA_NormBias",
+        # 'valid_geolocation': ValidGeolocation,
+        "valid_geolocation": None,
+    }
+
+    if name in catalog:
+        return catalog[name]
+
+
+def fix_procedure(cfg):
+    """If a test procedure is not defined, guess it from the name
+
+    For each variable it is defined a sequence of tests to apply. And each
+    test is based in one procedure, but multiple tests could be based on the
+    same procedure.
+
+    For instance, one could apply the GlobalRange with two different
+    thresholds, a lower one for flag 3 and a higher one for flag 4.
+    """
+
+    def fix_each_one(cfg):
+        for c in cfg:
+            procedure = guess_procedure(c, cfg[c])
+            if procedure is not None:
+                if cfg[c] is None:
+                    cfg[c] = {"procedure": procedure}
+                elif "procedure" not in cfg[c]:
+                    cfg[c]["procedure"] = procedure
+        return cfg
+
+    if "common" in cfg:
+        cfg["common"] = fix_each_one(cfg["common"])
+
+    if "variables" in cfg:
+        for v in cfg["variables"]:
+            cfg["variables"][v] = fix_each_one(cfg["variables"][v])
+
+    return cfg
